@@ -179,8 +179,7 @@ class Lookup:
 
     def __call__(self, word: str, *,
                  capitalization: bool = True,
-                 allow_nosuggest: bool = True,
-                 allow_break: bool = True) -> bool:
+                 allow_nosuggest: bool = True) -> bool:
         """
         The outermost word correctness check.
 
@@ -196,7 +195,6 @@ class Lookup:
 
             capitalization: if ``False``, check ONLY exactly this capitalization
             allow_nosuggest: if ``False``, don't consider correct words with ``NOSUGGEST`` flag
-            allow_break: if ``False``, don't try to break word by dashes and check separately
         """
 
         # The word is considered correct, if it can be deconstructed into a "good form" (the form
@@ -231,11 +229,6 @@ class Lookup:
         if is_correct(word):
             return True
 
-        # ``allow_break=False`` might've been passed from Suggest_ and mean we shouldn't try to
-        # break word.
-        if not allow_break:
-            return False
-
         # ``try_break`` recursively produces all possible lists of word breaking by break patterns
         # (like dashes).
         for parts in self.break_word(word):
@@ -654,8 +647,6 @@ class Lookup:
 
         root_flags = form.in_dictionary.flags
         all_flags = form.flags()
-        # # TODO: Should be guessed on dictionary loading
-        # root_capitalization = aff.casing.guess(form.in_dictionary.stem)
 
         # If the stem has NOSUGGEST flag, it shouldn't be considered an existing word when called
         # from ``Suggest`` (in other cases allow_nosuggest is True). This allows, for example, to
