@@ -3,7 +3,7 @@ The main "suggest correction for this misspelling" module.
 
 On a bird-eye view level, suggest does:
 
-* tries small word changes (remove letters, insert letters, swap letters) and checks (with the help
+* tries small word "edits" (remove letters, insert letters, swap letters) and checks (with the help
   of :mod:`lookup  <spylls.hunspell.algo.lookup>`) there are any valid ones
 * if no good suggestions found, tries "ngram-based" suggestions (calculating ngram-based distance to
   all dictionary words and select the closest ones), handled by
@@ -12,8 +12,8 @@ On a bird-eye view level, suggest does:
 
 Note that Spylls's implementation takes two liberties comparing to Hunspell's:
 
-1. In Hunspell, all permutations-based logic is run twice: first, checks if any of the permutated variants
-   is a valid non-compound word; then (if nothing good was found), for all the same permutations, checks
+1. In Hunspell, all edits-generation logic is run twice: first, checks if any of the edits
+   is a valid non-compound word; then (if nothing good was found), for all the same edits, checks
    if maybe it is a valid compound word. It is done this way because checking whether word is correct
    *not regarding compounding* is much faster. We ignore this optimization in the name of clarity
    of the algorithm -- and on the way make suggestions better in edge cases: when compound and non-compound
@@ -55,6 +55,8 @@ from spylls.hunspell.algo.capitalization import Type as CapType
 from spylls.hunspell.algo import ngram_suggest, phonet_suggest, permutations as pmt
 
 MAXPHONSUGS = 2
+MAXSUGGESTIONS = 15
+GOOD_EDITS = ['spaceword', 'uppercase', 'replchars']
 
 
 @dataclass
@@ -66,18 +68,12 @@ class Suggestion:
 
     #: Actual suggestion text
     text: str
-    #: Code specifying how suggestion was produced, useful for debugging, typically same as the method
-    #: of the permutation which led to this suggestion, like "badchar", "twowords", "phonet" etc.
-    source: str
-
-    #: If ``False``, then checking suggestion validity should be without trying to break it on dashes
-    #: (or similar chars, depending on the language). This is used, for example, to check "very good"
-    #: suggestions: "foobar" (misspeling) => "foo-bar" considered "very good" ONLY if the dictionary
-    #: contains "foo-bar" itself, not "foo" and "bar".
-    allow_break: bool = True
+    #: How suggestion was produced, useful for debugging, typically same as the method
+    #: of the editing which led to this suggestion, like "badchar", "twowords", "phonet" etc.
+    kind: str
 
     def __repr__(self):
-        return f"Suggestion[{self.source}]({self.text})"
+        return f"Suggestion[{self.kind}]({self.text})"
 
     def replace(self, **changes):
         return dataclasses.replace(self, **changes)
@@ -96,7 +92,7 @@ class MultiWordSuggestion:
 
     #: Whether those words are allowed to be joined by dash. We should disallow it if the multi-word
     #: suggestion was produced by :attr:`Aff.REP <spylls.hunspell.data.aff.Aff.REP>` table, see
-    #: :meth:`Suggest.good_permutations` for details.
+    #: :meth:`Suggest.edits` for details.
     allow_dash: bool = True
 
     def stringify(self, separator=' '):
@@ -129,14 +125,9 @@ class Suggest:
     .. automethod:: __call__
     .. automethod:: suggest_internal
 
-    **Permutation-based suggestions**
-
-    .. automethod:: very_good_permutations
-    .. automethod:: good_permutations
-    .. automethod:: questionable_permutations
-
-    **Other suggestions**
+    **Suggestion types**
 
+    .. automethod:: edits
     .. automethod:: ngram_suggestions
     .. automethod:: phonet_suggestions
     """
@@ -188,10 +179,9 @@ class Suggest:
 
         * generates possible misspelled word cases (for ex., "KIttens" in dictionary might've been
           'KIttens', 'kIttens', 'kittens', or 'Kittens')
-        * produces word permutations with :meth:`good_permutations`, :meth:`very_good_permutations` and
-          :meth:`questionable_permutations` (with the help of :mod:`permutations <spylls.hunspell.algo.permutations>`
-          module), checks them with :class:`Lookup <spylls.hunspell.algo.lookup.Lookup>`,
-          and decides if that's maybe enough
+        * produces word edits with :meth:`edits` (with the help of
+          :mod:`permutations <spylls.hunspell.algo.permutations>` module), checks them with
+          :class:`Lookup <spylls.hunspell.algo.lookup.Lookup>`, and decides if that's maybe enough
         * but if it is not (and if .aff settings allow), ngram-based suggestions are produced with
           :meth:`ngram_suggestions`, and phonetically similar suggestions with :meth:`phonet_suggestions`
 
@@ -203,8 +193,10 @@ class Suggest:
 
         # Whether some suggestion (permutation of the word) is an existing and allowed word,
         # just delegates to Lookup
-        def is_good_suggestion(word, capitalization=False, allow_break=True):
-            return self.lookup(word, allow_nosuggest=False, capitalization=capitalization, allow_break=allow_break)
+        def is_good_suggestion(word, capitalization=False):
+            # Note that instead of using Lookup's main method, we just see if there is any good forms
+            # of this exact word, avoiding ICONV and trying to break word by dashes.
+            return any(self.lookup.good_forms(word, capitalization=capitalization, allow_nosuggest=False))
 
         # For some set of suggestions, produces only good ones:
         def filter_suggestions(suggestions):
@@ -212,7 +204,7 @@ class Suggest:
                 # For multiword suggestion,
                 if isinstance(suggestion, MultiWordSuggestion):
                     # ...if all of the words is correct
-                    if all(is_good_suggestion(word, allow_break=False) for word in suggestion.words):
+                    if all(is_good_suggestion(word) for word in suggestion.words):
                         # ...we just convert it to plain text suggestion "word1 word2"
                         yield suggestion.stringify()
                         if suggestion.allow_dash:
@@ -220,7 +212,7 @@ class Suggest:
                             yield suggestion.stringify('-')
                 else:
                     # Singleword suggestion is just yielded if it is good
-                    if is_good_suggestion(suggestion.text, allow_break=suggestion.allow_break):
+                    if is_good_suggestion(suggestion.text):
                         yield suggestion
 
         # The suggestion is considered forbidden if there is ANY homonym in dictionary with flag
@@ -240,9 +232,9 @@ class Suggest:
         # Method is quite lengthy, but is nested because updates and reuses ``handled`` local var
         def handle_found(suggestion, *, check_inclusion=False):
             text = suggestion.text
+
             # If any of the homonyms has KEEPCASE flag, we shouldn't coerce it from the base form.
             # But CHECKSHARPS flag presence changes the meaning of KEEPCASE...
-
             if (self.aff.KEEPCASE and self.dic.has_flag(text, self.aff.KEEPCASE) and not
                     (self.aff.CHECKSHARPS and 'ß' in text)):
                 # Don't try to change text's case
@@ -267,7 +259,14 @@ class Suggest:
             if is_forbidden(text):
                 return
 
+            # Finally, OCONV table in .aff-file might specify what chars to replace in suggestions
+            # (for example, "'" to proper typographic "’", or common digraphs)
+            text = self.aff.OCONV(text) if self.aff.OCONV else text
+
             # If we already seen this suggestion, nothing to do
+            # Note that this should happen AFTER the OCONV: it sometimes changes the content significantly.
+            # For example, Dutch dictionary recodes "ij" into ligature (one character representing this
+            # two letters) to enforce proper capitalization (and decodes it back on OCONV).
             if text in handled:
                 return
 
@@ -278,10 +277,8 @@ class Suggest:
             if check_inclusion and any(previous.lower() in text.lower() for previous in handled):
                 return
 
+            # Remember we seen it
             handled.add(text)
-            # Finally, OCONV table in .aff-file might specify what chars to replace in suggestions
-            # (for example, "'" to proper typographic "’", or common digraphs)
-            text = self.aff.OCONV(text) if self.aff.OCONV else text
 
             # And here we are!
             yield suggestion.replace(text=text)
@@ -292,12 +289,9 @@ class Suggest:
         # if the word is "MsDonalds", good capitalizations (what it might've been in dictionary) are
         # "msdonalds" (full lowercase) "msDonalds" (first letter lowercased), or maybe "Msdonalds"
         # (only first letter capitalized). Note that "MSDONALDS" (it should've been all caps) is not
-        # produced as a possible good form, but checked separately in good_permutations
+        # produced as a possible good form, but checked separately in ``edits``
         captype, variants = self.aff.casing.corrections(word)
 
-        good = False
-        very_good = False
-
         # Check a special case: if it is possible that words would be possible to be capitalized
         # on compounding, then we check capitalized form of the word. If it is correct, that's the
         # only suggestion we ever need.
@@ -307,41 +301,73 @@ class Suggest:
                     yield from handle_found(Suggestion(capitalized.capitalize(), 'forceucase'))
                     return  # No more need to check anything
 
+        good_edits_found = False
+        enough_edits_found = False
+
         # Now, for all capitalization variant
         for idx, variant in enumerate(variants):
             # If it is different from original capitalization, and is good, we suggest it
             if idx > 0 and is_good_suggestion(variant):
                 yield from handle_found(Suggestion(variant, 'case'))
 
-            # Now check if any of the good permutations would be yielded
-            for suggestion in filter_suggestions(self.good_permutations(variant)):
+            # Now we take all possible edits of the word, and filter those that are correct
+            # words
+            for suggestion in filter_suggestions(self.edits(variant)):
+                # Then, for each of those correct suggestions,
+                # we run them through handle_found (which performs some final transformations and
+                # checks, and may handle 1 or 0 suggestions; 0 in the case it was already seen or
+                # is a forbidden word)
                 for res in handle_found(suggestion):
-                    # ...and if so, return them and set good flag
-                    good = True
+                    # yield it to user
                     yield res
+                    # remember if any "good" edits was found -- in this case we don't need
+                    # ngram suggestions
+                    good_edits_found = good_edits_found or (res.kind in GOOD_EDITS)
 
-            # Now check if any of the VERY good permutations would be yielded
-            # (yes, the order in hunspell is this: what we are calling here "very good permutations"
-            # is tried AFTER just "good" permutations; but they weight more, see below)
-            for suggestion in filter_suggestions(self.very_good_permutations(variant)):
-                for res in handle_found(suggestion):
-                    very_good = True
-                    yield res
+                    # We might break from the method immediately if the suggestion is to split word
+                    # with space, and this phrase is _in the dictionary as a whole_: "alot" => "a lot".
+                    # Then we don't need ANY other suggestions. This is a trick to enforce suggesting
+                    # to break commonly misspelled phrases withot oversuggesting
+                    if res.kind == 'spaceword':
+                        return
 
-            # ...if any _very_ good permutation was found, nothing to check anymore
-            if very_good:
-                return
+                    # Also we stop after 15 edits-based suggestions (but still may produce
+                    # ngram-based ones if edits-based were not particularly good),
+                    if len(handled) > MAXSUGGESTIONS:
+                        enough_edits_found = True
+                        break
+                if enough_edits_found:
+                    break
+            if enough_edits_found:
+                break
 
-            # ...but now we'll check "questionable" permutations (which might produce quite unlikely words) --
-            # even if "good" permutations were present; but good permutations would be earlier in the list
-            for suggestion in filter_suggestions(self.questionable_permutations(variant)):
-                yield from handle_found(suggestion)
-
-        if very_good or good:
+        if good_edits_found:
             return
 
-        # If there was no "good" or "very good" permutations that were valid words, we might try
-        # ngram-based suggestion algorithm: it is slower, but able to find severely misspelled words
+        # One additional pass at suggesting: if the word contains dashes, and there were no dash-containing
+        # suggestions, we should try to spellcheck it separately... But only one misspelled part is allowed.
+        if '-' in word and not any('-' in sug for sug in handled):
+            chunks = word.split('-')
+            for idx, chunk in enumerate(chunks):
+                # If this chunk of the word is misspelled...
+                if not is_good_suggestion(chunk):
+                    # ...try all suggestions for this separate chunk
+                    for sug in self(chunk):
+                        candidate = '-'.join([*chunks[:idx], sug, *chunks[idx+1:]])
+                        # And check if the whole word with this chunk replaced is a good word. Note that
+                        # we use lookup's main method here, which will also break it into words by
+                        # dashes. It is done (instead of just checking chunk-by-chunk), because there
+                        # are ambiguities how to split: if we are fixing "quas-Afro-American"->"quasi-Afro-American",
+                        # and "Afro-American" is present in the dictionary as a whole word, it is better
+                        # to delegate search for the right breaking to the Lookup.
+                        if self.lookup(candidate):
+                            yield Suggestion(candidate, 'dashes')
+                    # after only one misspelled chunk, we stop the search anyways, whether we found some
+                    # sugestions or not
+                    break
+
+        # If there was no good edits that were valid words, we might try
+        # ngram-based suggestion algorithm: it is slower, but able to correct severely misspelled words
 
         ngrams_seen = 0
         for sug in self.ngram_suggestions(word, handled=handled):
@@ -362,36 +388,21 @@ class Suggest:
             if phonet_seen >= MAXPHONSUGS:
                 break
 
-    def very_good_permutations(self, word: str) -> Iterator[Suggestion]:
+    def edits(self, word: str) -> Iterator[Union[Suggestion, MultiWordSuggestion]]:
         """
-        "Very good" suggestions: suggest to split word ("alot" => "a lot"), but for now only yield
-        them as a *singular* word suggestion: if the dictionary has *exact* entry "a lot", it would
-        be considered correct.
+        Produces all possible word edits in a form of :class:`Suggestion` or :class:`MultiWordSuggestion`.
+        Note that:
+
+        * order is important, that's the order user will receive the suggestions (and the further the
+          suggestion type in the order, the more probably it would be dropped due to suggestion count
+          limit)
+        * suggestion "source" tag is important: :meth:`suggest_internal` uses it to distinguish between
+          good and questionble edits (if there were any good ones, ngram suggestion wouldn't
+          be used)
 
         Args:
             word: Word to mutate
         """
-
-        for words in pmt.twowords(word):
-            yield Suggestion(' '.join(words), 'spaceword')
-
-            if self.use_dash:
-                # "alot" => "a-lot", but make sure it would be checked as a whole word (see allow_break
-                # usage in Lookup)
-                yield Suggestion('-'.join(words), 'spaceword', allow_break=False)
-
-    def good_permutations(self, word: str) -> Iterator[Union[Suggestion, MultiWordSuggestion]]:
-        """
-        Good permutations (that produces words not very different from the initial one):
-
-        * uppercase word;
-        * replacements via :attr:`Aff.REP <spylls.hunspell.data.aff.Aff.REP>`-table (may produce
-          :class:`MultiWordSuggestion` if REP table included replacement with a space)
-
-        Args:
-            word: Word to mutate
-        """
-
         # suggestions for an uppercase word (html -> HTML)
         yield Suggestion(self.aff.casing.upper(word), 'uppercase')
 
@@ -412,27 +423,12 @@ class Suggest:
             else:
                 yield Suggestion(suggestion, 'replchars')
 
-    def questionable_permutations(self, word: str) -> Iterator[Union[Suggestion, MultiWordSuggestion]]:
-        """
-        Permutations that are producing suggestions further from the original word:
+        for words in pmt.twowords(word):
+            yield Suggestion(' '.join(words), 'spaceword')
 
-        * replacements by :attr:`Aff.MAP <spylls.hunspell.data.aff.Aff.MAP>` table (very similar chars, like ``aáã``)
-        * adjacent char swapping
-        * non-adjacent char swapping
-        * replacements by :attr:`Aff.KEY <spylls.hunspell.data.aff.Aff.KEY>` table (chars that are close on keyboard)
-        * removal of characters
-        * insertion of characters
-        * moving of singular character
-        * replacement of chars by all chars in alphabet
-        * removal of possible two-char doubling ("vacacation => vacation")
-        * splitting of word into two
-
-        Order is important: As the whole ``Suggest`` produces generator, client code may consume it
-        one-by-one, so the first suggested means more likely.
-
-        Args:
-            word: Word to mutate
-        """
+            if self.use_dash:
+                # "alot" => "a-lot"
+                yield Suggestion('-'.join(words), 'spaceword')
 
         # MAP in aff file specifies related chars (for example, "ïi"), and mapchars produces all
         # changes of the word with related chars replaced. For example, "naive" produces "naïve".
